## Copyright (c) 2012 Miguel Bazdresch
##
## Permission is hereby granted, free of charge, to any person obtaining a
## copy of this software and associated documentation files (the "Software"),
## to deal in the Software without restriction, including without limitation
## the rights to use, copy, modify, merge, publish, distribute, sublicense,
## and/or sell copies of the Software, and to permit persons to whom the
## Software is furnished to do so, subject to the following conditions:
##
## The above copyright notice and this permission notice shall be included in
## all copies or substantial portions of the Software.
##
## THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
## IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
## FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
## AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
## LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
## FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
## DEALINGS IN THE SOFTWARE.
#
# Draw the Mandelbrot set
#
# Inspired by the gtk-server fractal demo, by Peter van Eerten
#

load("gtk.jl")

function draw_fractal(xmin,xmax,ymin,ymax,maxiters)
    if xmax-xmin != ymax-ymin
        gtk_exit(fifo)
        error("Only square ranges are supported.")
    end
    # parameters
    ps = (xmax-xmin)/480      # euclidean distance between pixels

    gtk_send(fifo,"gdk_color_parse \#000000 $color")
    gtk_send(fifo,"gdk_gc_set_rgb_fg_color $gcn $color")
    gtk_send(fifo,"gdk_draw_layout $pix $gcn 10 480 $startt")
    gtk_send(fifo,"gtk_widget_queue_draw $image")
    gtk_send(fifo,"gtk_server_callback update")

    y = ymax
    for j = 0:479
        x = xmin
        for k = 0:479
            r = 0
            i = 0
            iters = 0
            while (r*r+i*i < 4) && (iters < maxiters)
                iters += 1
                rt = r*r - i*i + x
                i = 2r*i + y
                r = rt
            end
            # color depends on iters
            qcolor = int(floor(256*iters/maxiters))
            pcolor = "$(hex(qcolor,2))"^3
            gtk_send(fifo,"gdk_color_parse \#$pcolor $color")
            gtk_send(fifo,"gdk_gc_set_rgb_fg_color $gcn $color")
            gtk_send(fifo,"gdk_draw_point $pix $gcn $k $j")
            result = gtk_send(fifo,"gtk_server_callback update")
            if result == exit_button || result == win
                return "exit"
            end
            x += ps
        end
        gtk_send(fifo,"gtk_widget_queue_draw $image")
        gtk_send(fifo,"gtk_server_callback update")
        y -= ps
    end
    # wipe wait text
    gtk_send(fifo,"gdk_color_parse \#ffffff $color")
    gtk_send(fifo,"gdk_gc_set_rgb_fg_color $gcn $color")
    gtk_send(fifo,"gdk_draw_rectangle $pix $gcn 1 10 480 110 25")
    # Tell drawing is ready
    gtk_send(fifo,"gdk_color_parse \#000000 $color")
    gtk_send(fifo,"gdk_gc_set_rgb_fg_color $gcn $color")
    gtk_send(fifo,"gdk_draw_layout $pix $gcn 10 480 $ready")
    gtk_send(fifo,"gtk_widget_queue_draw $image")
    return "ok"
end

fifo=gtk_init()
gtk_send(fifo,"gtk_init NULL NULL")
win = gtk_send(fifo,"gtk_window_new 0")
gtk_send(fifo,"gtk_window_set_title $win 'Julia GTK Mandelbrot generator'")
gtk_send(fifo,"gtk_widget_set_size_request $win 480 550")
gtk_send(fifo,"gtk_window_set_position $win 1")
gtk_send(fifo,"gtk_window_set_resizable $win 0")
# Create widget to display image
image = gtk_send(fifo,"gtk_image_new")
# Create eventbox to catch mouseclick
ebox = gtk_send(fifo,"gtk_event_box_new")
gtk_send(fifo,"gtk_container_add $ebox $image")
# Separator
sep = gtk_send(fifo,"gtk_hseparator_new")
# Action button
action_button = gtk_send(fifo,"gtk_button_new_with_label Draw")
gtk_send(fifo,"gtk_widget_set_size_request $action_button 75 30")
# Clear button
clear_button = gtk_send(fifo,"gtk_button_new_with_label Clear")
gtk_send(fifo,"gtk_widget_set_size_request $clear_button 75 30")
# Exit button
exit_button = gtk_send(fifo,"gtk_button_new_with_label Exit")
gtk_send(fifo,"gtk_widget_set_size_request $exit_button 75 30")
# Now arrange widgets on window using boxes
hbox = gtk_send(fifo,"gtk_hbox_new 0 0")
gtk_send(fifo,"gtk_box_pack_start $hbox $clear_button 0 0 1")
gtk_send(fifo,"gtk_box_pack_start $hbox $action_button 0 0 1")
gtk_send(fifo,"gtk_box_pack_end $hbox $exit_button 0 0 1")
vbox = gtk_send(fifo,"gtk_vbox_new 0 0")
gtk_send(fifo,"gtk_box_pack_start $vbox $ebox 0 0 1")
gtk_send(fifo,"gtk_box_pack_start $vbox $sep 0 0 1")
gtk_send(fifo,"gtk_box_pack_end $vbox $hbox 0 0 1")
gtk_send(fifo,"gtk_container_add $win $vbox")
# Show all widgets
gtk_send(fifo,"gtk_widget_show_all $win")
# Create the pixmap
gdkwin = gtk_send(fifo,"gtk_widget_get_parent_window $image")
pix = gtk_send(fifo,"gdk_pixmap_new $gdkwin 480 500 -1")
gcn = gtk_send(fifo,"gdk_gc_new $pix")
gtk_send(fifo,"gtk_image_set_from_pixmap $image $pix NULL")
# Allocate memory with some random widget for GdkColor
color = gtk_send(fifo,"gtk_frame_new NULL")
# Now set foreground and backgroundcolors to WHITE
gtk_send(fifo,"gdk_color_parse \#ffffff $color")
gtk_send(fifo,"gdk_gc_set_rgb_bg_color $gcn $color")
gtk_send(fifo,"gdk_gc_set_rgb_fg_color $gcn $color")
# Clear the complete pixmap with WHITE
gtk_send(fifo,"gdk_draw_rectangle $pix $gcn 1 0 0 480 500")
# Set color to BLACK
gtk_send(fifo,"gdk_color_parse \#000000 $color")
gtk_send(fifo,"gdk_gc_set_rgb_fg_color $gcn $color")
# Put some text on the canvas
layout = gtk_send(fifo,"gtk_widget_create_pango_layout $image 'Draw a fractal with Julia!'")
gtk_send(fifo,"gdk_draw_layout $pix $gcn 120 480 $layout")
# Define start and finishing text
startt = gtk_send(fifo,"gtk_widget_create_pango_layout $image 'Please wait...'")
ready = gtk_send(fifo,"gtk_widget_create_pango_layout $image 'Drawing ready.'")
# Update the image widget with the pixmap
gtk_send(fifo,"gtk_widget_queue_draw $image")

# main loop
result = 0
while result != win && result != exit_button
    result = gtk_send(fifo,"gtk_server_callback wait")
    if result == action_button
        s = draw_fractal(-2.3,0.7,-1.5,1.5,100)
        if s == "exit"
            break
        end
    elseif result == clear_button
        gtk_send(fifo,"gdk_color_parse \#ffffff $color")
        gtk_send(fifo,"gdk_gc_set_rgb_fg_color $gcn $color")
        gtk_send(fifo,"gdk_draw_rectangle $pix $gcn 1 0 0 480 500")
        gtk_send(fifo,"gdk_color_parse \#000000 $color")
        gtk_send(fifo,"gdk_gc_set_rgb_fg_color $gcn $color")
        gtk_send(fifo,"gdk_draw_layout $pix $gcn 120 480 $layout")
        gtk_send(fifo,"gtk_widget_queue_draw $image")
    end
end

gtk_exit(fifo)
