## Copyright (c) 2012 Miguel Bazdresch
##
## Permission is hereby granted, free of charge, to any person obtaining a
## copy of this software and associated documentation files (the "Software"),
## to deal in the Software without restriction, including without limitation
## the rights to use, copy, modify, merge, publish, distribute, sublicense,
## and/or sell copies of the Software, and to permit persons to whom the
## Software is furnished to do so, subject to the following conditions:
##
## The above copyright notice and this permission notice shall be included in
## all copies or substantial portions of the Software.
##
## THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
## IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
## FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
## AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
## LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
## FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
## DEALINGS IN THE SOFTWARE.
#
# Draw the Mandelbrot set
#
# Inspired by the gtk-server fractal demo, by Peter van Eerten
#

load("gtk.jl")

function draw_fractal(xmin,xmax,ymin,ymax,maxiters)
    if xmax-xmin != ymax-ymin
        gtk_exit(fifo)
        error("Only square ranges are supported.")
    end
    # parameters
    ps = (xmax-xmin)/480      # euclidean distance between pixels
    # array to store fractal
    #fname = "/tmp/fractalhug.$(randstring(5)).ppm"
    fname = "test.ppm"
    fid = open(fname,"w")
    mandel = zeros(Uint8,480,480*3)

    # clear canvas
    gtk_send(fifo,"u_draw $canvas")
    gtk_send(fifo,"u_square #FFF 0 0 480 480 1")
    gtk_send(fifo,"u_draw $sbar")
    gtk_send(fifo,"u_square #FFF 0 0 480 23 1")
    gtk_send(fifo,"u_out \"'Please wait... Click Exit to terminate.'\" #000 #FFF 3 3")

    y = ymax
    for j = 1:480
        x = xmin
        for k = 1:3:480*3
            r = 0
            i = 0
            iters = 0
            while (r*r+i*i < 4) && (iters < maxiters)
                iters += 1
                rt = r*r - i*i + x
                i = 2r*i + y
                r = rt
            end
            # Color depends on iters.
            # This is the simplest mapping -- to get more visually
            #  striking results, one needs more complex mappings.
            color = int(floor(256*iters/maxiters))
            # At the moment we plot in grayscale -- we could use PGM instead
            # of PPM.
            mandel[j,k] = color
            mandel[j,k+1] = color
            mandel[j,k+2] = color
            result = gtk_send(fifo,"u_event arg")
            if result == exit_button || result == win
                return "exit"
            end
            x += ps
        end
        y -= ps
    end
    # write PPM file
    write(fid,"P6\n480 480\n255\n")
    write(fid,mandel')
    close(fid)
    gtk_send(fifo,"u_draw $canvas")
    gtk_send(fifo,"u_image $fname")
    gtk_send(fifo,"u_draw $sbar")
    gtk_send(fifo,"u_square #FFF 0 0 480 23 1")
    gtk_send(fifo,"u_out \"'Done. Click Clear to erase fractal.'\" #000 #FFF 3 3")
    system("rm $fname")
    return "ok"
end

fifo=gtk_init()
win = gtk_send(fifo,"u_window \"'Julia GTK Mandelbrot generator'\" 490 550")
# Create and position canvas to display image
frame = gtk_send(fifo,"u_frame 483 483")
gtk_send(fifo,"u_attach $win $frame 4 4")
canvas = gtk_send(fifo,"u_canvas 480 480")
gtk_send(fifo,"u_attach $win $canvas 5 5")
# Create widget to display status (HUG has no support for status bar)
framesb = gtk_send(fifo,"u_frame 483 23")
gtk_send(fifo,"u_attach $win $framesb 4 490")
sbar = gtk_send(fifo,"u_canvas 480 20")
gtk_send(fifo,"u_attach $win $sbar 5 491")
# Action button
action_button = gtk_send(fifo,"u_button \"'Draw'\" 75 30")
gtk_send(fifo,"u_attach $win $action_button 5 515")
# Clear button
clear_button = gtk_send(fifo,"u_button \"'Clear'\" 75 30")
gtk_send(fifo,"u_attach $win $clear_button 85 515")
# Exit button
exit_button = gtk_send(fifo,"u_button \"'Exit'\" 75 30")
gtk_send(fifo,"u_attach $win $exit_button 400 515")
# Put some text on the status bar
gtk_send(fifo,"u_draw $sbar")
gtk_send(fifo,"u_out \"'Click the Draw button to generate the Mandelbrot set.'\" #000 #FFF 3 3")

# main loop
result = 0
while result != win && result != exit_button
    result = gtk_send(fifo,"u_event")
    if result == action_button
        s = draw_fractal(-2.3,0.7,-1.5,1.5,100)
        if s == "exit"
            break
        end
    elseif result == clear_button
        gtk_send(fifo,"u_square #FFF 0 0 480 480 1")
        gtk_send(fifo,"u_draw $sbar")
        gtk_send(fifo,"u_square #FFF 0 0 480 23 1")
        gtk_send(fifo,"u_out \"'Click the Draw button to generate the Mandelbrot set.'\" #000 #FFF 3 3")
    end
end

gtk_exit(fifo)
